/**
* C_lib Copyright (C) 2021 Wael Karram.
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, version 3 of the License.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

/* This file contains the function implementation for safe string copy. */
/* Include guards + includes. */
#ifndef STRSCPY_C
#define STRSCPY_C
/* Include statements. */
#include "strscpy.h"
#endif //STRSCPY_C

/**
* This function takes a destination buffer, a source buffer and a length.
* Copies the source into the destination, stopping when length is reached or end of destination (whichever is shorter).
* The copied string is always null-terminated when possible (might cause truncation).
* Will return the length copied (and whether it overflowed or not).
*/
char *strscpy(char *restrict dest, const char* restrict src, size_t n) {
	/* Copy using memcpy. */
	char *last = memccpy(dest, src, NULL_CHAR, n);

	/* Check if the copy succeeded, and make sure there is a NULL-terminator at the end. */
	if (!last)
		*(dest + (--n)) = NULL_CHAR;

	/* Return the pointer from memcpy. */
	return last;
}
