/* Include statements. */
#include <stdio.h>
#include <assert.h>
#include <time.h>
#include <stdlib.h>
#include "../misc_algorithms/search/binary_search.h"
#include "../utils/time_function.h"

/* Define constants. */
#define PRINT_INFO_TESTING_START		"Starting tests.\n"
#define PRINT_INFO_TESTING_END			"Done testing.\n"
#define SOME_TEST_FAILED			"Some of the tests have failed.\n"
#define TESTED_FUNCTION_NAME			"binary_search()"
#define TESTING_ALLOCATION_ERROR		"Testing allocation error, free some memory and try again later. \n"
#define RUNNING_TIME				"Ran in %lf seconds.\n"
#define SORTED_INT_ARRAY_FIRST			{1, 2, 3, 4, 5, 6, 7, 8, 9, 10}
#define SORTED_ARRAY_FIRST_LENGTH		10
#define FIRST_ARRAY_FIRST_SEARCH_VALUE		5
#define FIRST_ARRAY_FIRST_SEARCH_VALUE_INDEX	4
#define FIRST_ARRAY_SECOND_SEARCH_VALUE		8
#define FIRST_ARRAY_SECOND_SEARCH_VALUE_INDEX	7
#define SORTED_INT_ARRAY_SECOND			{0, 2, 3, 4, 5, 6, 7, 8, 8, 10}
#define SORTED_ARRAY_SECOND_LENGTH		10
#define SECOND_ARRAY_FIRST_SEARCH_VALUE		2
#define SECOND_ARRAY_FIRST_SEARCH_VALUE_INDEX	1
#define SECOND_ARRAY_SECOND_SEARCH_VALUE	8
#define SECOND_ARRAY_SECOND_SEARCH_VALUE_INDEX	7
#define ALLOCATION_ERROR_CODE			75

/* Define functions. */
int integer_comparison_function(const void *first, const void* second);
int test_binary_search();

/* Comparison function. */
int integer_comparison_function(const void *first, const void* second) {
	/* Cast and compare. */
	int first_value, second_value, comparison_result;
	first_value = *((int*) first);
	second_value = *((int*) second);
	// Check in two stages.
	if (first_value == second_value)
		return 0;
	return (first_value < second_value) ? -1 : 1;
}

/* Starting point. */
int main() {
	printf(PRINT_INFO_TESTING_START);
	if (test_binary_search()) {
		printf(SOME_TEST_FAILED);
	}
	printf(PRINT_INFO_TESTING_END);
}

/* Testing function. */
int test_binary_search() {
	/* Local variables. */
	int index, value;
	clock_t start, end;
	function_run_time *runtime;

	/* Declare the arrays. */
	int first[SORTED_ARRAY_FIRST_LENGTH] = SORTED_INT_ARRAY_FIRST;
	int second[SORTED_ARRAY_SECOND_LENGTH] = SORTED_INT_ARRAY_SECOND;

	/* Attempt to allocate the function run time struct. */
	runtime = (function_run_time*) malloc(sizeof(function_run_time));
	if (runtime == NULL) {
		/* TODO: Check why the sanitizer complains of a leak. */
		printf(TESTING_ALLOCATION_ERROR);
		exit(ALLOCATION_ERROR_CODE);
	}

	/* Set the function's name. */
	runtime->name = TESTED_FUNCTION_NAME;

	/* Run tests on the first array. */
	value = FIRST_ARRAY_FIRST_SEARCH_VALUE;
	/* Timestamp. */
	start = clock();
	index = binary_search(&first, &value, integer_comparison_function, SORTED_ARRAY_FIRST_LENGTH, sizeof(int));
	/* Timestamp. */
	end = clock();
	/* Record and print. */
	runtime->time = end - start;
	printf(RUNNING_TIME, (((double)(runtime->time)) / ((double) CLOCKS_PER_SEC)));
	assert(index == FIRST_ARRAY_FIRST_SEARCH_VALUE_INDEX);
	value = FIRST_ARRAY_SECOND_SEARCH_VALUE;
	/* Timestamp. */
	start = clock();
	index = binary_search(&first, &value, integer_comparison_function, SORTED_ARRAY_FIRST_LENGTH, sizeof(int));
	/* Timestamp. */
	end = clock();
	/* Record and print. */
	runtime->time = end - start;
	printf(RUNNING_TIME, (((double)(runtime->time)) / ((double) CLOCKS_PER_SEC)));
	assert(index == FIRST_ARRAY_SECOND_SEARCH_VALUE_INDEX);

	/* Run tests on the second array. */
	value = SECOND_ARRAY_FIRST_SEARCH_VALUE;
	/* Timestamp. */
	start = clock();
	index = binary_search(&first, &value, integer_comparison_function, SORTED_ARRAY_SECOND_LENGTH, sizeof(int));
	/* Timestamp. */
	end = clock();
	/* Record and print. */
	runtime->time = end - start;
	printf(RUNNING_TIME, (((double)(runtime->time)) / ((double) CLOCKS_PER_SEC)));
	assert(index == SECOND_ARRAY_FIRST_SEARCH_VALUE_INDEX);
	value = SECOND_ARRAY_SECOND_SEARCH_VALUE;
	/* Timestamp. */
	start = clock();
	index = binary_search(&first, &value, integer_comparison_function, SORTED_ARRAY_SECOND_LENGTH, sizeof(int));
	/* Timestamp. */
	end = clock();
	/* Record and print. */
	runtime->time = end - start;
	printf(RUNNING_TIME, (((double)(runtime->time)) / ((double) CLOCKS_PER_SEC)));
	assert(index == SECOND_ARRAY_SECOND_SEARCH_VALUE_INDEX);

	/* Free the measurement struct. */
	free(runtime);

	/* Done. */
	return 0;
}

