/* Include statements. */
#include <stdio.h>
#include <assert.h>
#include <time.h>
#include <stdlib.h>
#include "../sorting_algorithms/quicksort/quicksort.c"
/* Include the struct to store the measurements and function name. */
#include "../utils/time_function.h"
/* For easier debugging. */
#ifdef DEBUG
#include "../utils/array_print.h"
#else
#define print_integer_array(result, length)
#endif

/* TODO: Check the timing code, it is currently printing only zero. */

/* Define constants. */
#define PRINT_INFO_TESTING_START		"Starting tests.\n"
#define PRINT_INFO_TESTING_END			"Done testing.\n"
#define SOME_TEST_FAILED			"Some test failed!\n"
#define TESTED_FUNCTION_NAME			"quicksort()"
#define TESTING_ALLOCATION_ERROR		"Testing allocation error, free some memory and try again later.\n"
#define TESTING_WITH_COPY			"Testing with array copying.\n"
#define TESTING_NO_COPY				"Testing without array copying.\n"
#define RUNNING_TIME				"Ran in %lf seconds.\n"
#define ARRAY_START_POINT			0
#define ARRAY_LENGTH				15
#define ALLOCATION_ERROR_CODE			75

/* Constants. */
const int unsorted[ARRAY_LENGTH] = {3, 5, 1, 2, 6, 4, 9, 7, 11, 10, 8, 15, 13, 14, 12};
const int sorted[ARRAY_LENGTH] = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15};

/* Define functions. */
int comparison_function_int(const void *first, const void *second);
int test_array_sort();

/* Starting point. */
int main() {
	printf(PRINT_INFO_TESTING_START);
	if (test_array_sort()) {
		printf(SOME_TEST_FAILED);
	}
	printf(PRINT_INFO_TESTING_END);
}

/* This function is used to compare two integers (through pointers). */
int comparison_function_int(const void *first, const void *second) {
	return (*((int*)(first)) >= *((int*)(second)));
}

/* This function does the actual testing. */
int test_array_sort() {
	/* Local variables. */
	clock_t start, end;
	function_run_time *runtime;
	int *result;
	int i;

	/* Attempt to allocate the struct. */
	runtime = (function_run_time*) malloc(sizeof(function_run_time));
	if (runtime == NULL) {
		printf(TESTING_ALLOCATION_ERROR);
		exit(ALLOCATION_ERROR_CODE);
	}

	/* Attempt to copy the array. */
	result = (int*) malloc(ARRAY_LENGTH * sizeof(int));
	/* Check for allocation failure. */
	if (result == NULL) {
		printf(TESTING_ALLOCATION_ERROR);
		free(runtime);
		exit(ALLOCATION_ERROR_CODE);
	}
	/* Copy. */
	for (i = 0; i < ARRAY_LENGTH; i++)
		result[i] = unsorted[i];

	/* For debugging purposes. */
	print_integer_array(result, ARRAY_LENGTH);

	/* Set the timestamp. */
	start = clock();
	/* Run. */
	result = (int*) quick_sort(((void*)result), comparison_function_int, ARRAY_LENGTH, sizeof(int));
	/* Set another timestamp. */
	end = clock();

	/* Print start. */
	printf(TESTING_WITH_COPY);

	/* Record the results. */
	runtime->name = TESTED_FUNCTION_NAME;
	runtime->time = end - start;

	/* Print the results. */
	printf(RUNNING_TIME, (((double)(runtime->time))/((double)(CLOCKS_PER_SEC))));

	/* Free the struct. */
	free(runtime);

	/* For debugging purposes. */
	print_integer_array(result, ARRAY_LENGTH);

	/* Check if the array is sorted. */
	for (i = 0; i < ARRAY_LENGTH; i++)
		assert(result[i] == sorted[i]);

	/* Free the result. */
	free(result);

	/* Ran successfully. */
	return 0;
}
