/* Include statements. */
#include <stdio.h>
#include <assert.h>
#include "../stack/stack.c"

/* Define constants. */
#define PRINT_INFO_TESTING_START	"Starting tests.\n"
#define PRINT_INFO_TESTING_END		"Done testing.\n"
#define SOME_TEST_FAILED		"Recheck the tests, one of them failed!\n"
#define TEST_CODE_FAILURE		"Error in test code, rerun!\n"
#define INITIAL_STACK_SIZE		0
#define STACK_SIZE_ONE			1
#define STACK_SIZE_TWO			2
#define STACK_SIZE_LONG_TEST		4096


/* Define functions. */
node_t* generate_random_node();
int test_stack();

/* Starting point. */
int main() {
	printf(PRINT_INFO_TESTING_START);
	if(test_stack()) {
		printf(TEST_CODE_FAILURE);
	}
	printf(PRINT_INFO_TESTING_END);
}

/* This function generates a random number and puts it in a newly allocated node. */
/* Caller has to free memory afterwards, returns NULL on failure. */
node_t* generate_random_node() {
	/* The node to return. */
	node_t* result;
	int *value;

	/* Attempt to allocate the node. */
	result = (node_t*) malloc(sizeof(node_t));
	if (result == NULL)
		return NULL;
	value = (int*) malloc(sizeof(int));
	if (value == NULL) {
		free(result);
		return NULL;
	}
	*value = random();

	/* Store and return. */
	result->next = NULL;
	result->data = value;
	return result;
}

/* This function tests the workings of the stack structure. */
/* Returns 0 on success. */
int test_stack() {
	/* Local variables. */
	stack_t *stack;
	node_t *node;
	int* value;

	/* Attempt to initialize the stack. */
	stack = initialize_stack();
	assert(stack != NULL);
	assert(stack->size == INITIAL_STACK_SIZE);
	assert(stack->head == NULL);

	/* Allocate and push a node. */
	node = generate_random_node();
	if (node == NULL) {
		printf(TEST_CODE_FAILURE);
		return 1;
	}
	if (!push(stack, node)) {
		printf(SOME_TEST_FAILED);
		return 1;
	}
	assert(stack->size == STACK_SIZE_ONE);
	assert(stack->head != NULL);
	assert(stack->head->next == NULL);

	/* Attempt to use the built-in node creation and push function. */
	value = (int*) malloc(sizeof(int));
	if (value == NULL) {
		printf(SOME_TEST_FAILED);
		return 1;
	}
	*value = 5; /* Random, chosen by fair dice roll. */
	if (!push_data(stack, value)) {
		printf(SOME_TEST_FAILED);
		return 1;
	}
	assert(stack->size == STACK_SIZE_TWO);
	assert(stack->head != NULL);
	assert(stack->head->next != NULL);
	assert(stack_size(stack) == STACK_SIZE_TWO);

	/* Attempt to free. */
	free_stack_data(&stack, NULL);
	assert(stack == NULL);

	/* Free the node. */
	//free(node);

	/* Success. */
	return 0;
}
