/* Include statements. */
#include <stdio.h>
#include <assert.h>
#include "../vector/vector.c"

/* Define constants. */
#define PRINT_INFO_TESTING_START	"Starting tests.\n"
#define PRINT_INFO_TESTING_END		"Done testing.\n"
#define SOME_TEST_FAILED		"Recheck the tests, one of them failed!\n"
#define TEST_CODE_FAILURE		"Error in test code, rerun!\n"
#define INITIAL_VECTOR_SIZE		8
#define VECTOR_SIZE_ONE			1
#define VECTOR_SIZE_TWO			2
#define VECTOR_SIZE_LONG_TEST		4096
#define VECTOR_LAST_ELEMENT_EMPTY_INDEX	0


/* Define functions. */
int test_vector();

/* Starting point. */
int main() {
	printf(PRINT_INFO_TESTING_START);
	if(test_vector()) {
		printf(TEST_CODE_FAILURE);
	}
	printf(PRINT_INFO_TESTING_END);
}

/* This function tests the workings of the vector structure. */
/* Returns 0 on success. */
int test_vector() {
	/* Local variables. */
	vector_t *vector;
	int* value;

	/* Attempt to initialize the vector. */
	vector = initialize_vector();
	assert(vector != NULL);
	assert(vector->length == INITIAL_VECTOR_SIZE);
	assert(vector->array != NULL);
	assert(vector->last_element == VECTOR_LAST_ELEMENT_EMPTY_INDEX);
	assert(vector->empty);

	/* Allocate and insert a value. */
	value = (int*) malloc(sizeof(int));
	if (value == NULL) {
		printf(SOME_TEST_FAILED);
		return 1;
	}
	*value = 3; /* Random, chosen by fair dice roll. */
	if (!append_to_vector(vector, value)) {
		printf(SOME_TEST_FAILED);
		return 1;
	}
	assert(vector->length == INITIAL_VECTOR_SIZE);
	assert(vector->array != NULL);
	assert(vector->last_element == VECTOR_LAST_ELEMENT_EMPTY_INDEX);
	assert(!vector->empty);

	/* Attempt to free. */
	free_vector(vector, NULL);

	/* Free the data. */
	free(value);

	/* Success. */
	return 0;
}
