/*
* C_lib Copyright (C) 2021 Wael Karram.
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, version 3 of the License.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

/* This file defines a simple binary tree and related functions thereof. */
#include <stdlib.h>
#ifndef BINARY_TREE_H
#define BINARY_TREE_H
#include "../../nodes/tree_node.h"

/* Define constants. */
/* Return code for success. */
#define TREE_SUCCESS_CODE		0
/* Return code for failure. */
#define TREE_FAILURE_CODE		1
/* Return code for comparison, smaller. */
#define TREE_COMPARISON_SMALLER_THAN	-1
/* Return code for comparison, equal. */
#define TREE_COMPARISON_EQUAL_TO	0
/* Return code for comparison, larger. */
#define TREE_COMPARISON_LARGER_THAN	1

/* Define a max macro for use within the depth function. */
#define max(a,b) \
({ __typeof__ (a) _a = (a); \
   __typeof__ (b) _b = (b); \
   _a > _b ? _a : _b; })

/* Define all the functions. */
/* This function initializes a single node. */
tree_node_t* initialize_tree_node();
/* This function initializes a single node and attempts to store the given data within. */
tree_node_t* initialize_tree_node_store(void *data);
/* This function links a node to the right. */
int link_tree_node_right(tree_node_t *parent, tree_node_t *child);
/* This function links a node to the left. */
int link_tree_node_left(tree_node_t *parent, tree_node_t *child);
/* This function frees a whole tree - not including data. */
void free_tree(tree_node_t *root);
/* This function frees a whole tree, including data. */
void free_tree_data(tree_node_t *root, void (*free_function)(const void*));
/* This function inserts data to the tree. */
int insert_node_into_tree(tree_node_t *root, tree_node_t *to_insert, int (*comparison_function)(const void*, const void*));
/* This function finds the tree's depth (at the deepest leaf). */
size_t find_tree_depth(tree_node_t *root);
#endif /* BINARY_TREE_H */
